#include <stdio.h>
#include <math.h>
#include "mainloop.h"
#include "matrix.h"
#include "ray.h"
#include "surfaces.h"
#include "lights.h"
#include "ppmutil.h"
#include "globals.h"
#include "surfprops.h"
#include "csg.h"

PPMrast *picture, *squares, *eye, *cylwall, *sphwall;
Scene *scene;


void DrawScene() {
  ray *a_ray;
  screen a_screen;
  Surface *surface, *s2;
  v3 *a_photon, *unitleft, *unitright, *unitupleft, *unitupright, *backleft, *backright, *backupleft,
				*backupright;
  int triples_per_line, shelloop;
  LightList *lightlist;
  LightSource *pointsource;
  Sphere *sphere;
  Plane *p2, *plane;
  real radmult, pitch;
  Cylinder *cyl;
  SurfProps *surfprop;
  v3 *a_color;
  CSGnode *csg2, *csgnode, *notnode, *notshell, *pyramid;

  a_screen.init();
// sloppy init for test
  pointsource = new LightSource(pointTag, v3(9.0, -9.0, -9.0), 1.0, 
				v3(0.70, 0.7, 0.70));
  lightlist = new LightList(NULL);
  lightlist->Add(pointsource);
  scene = new Scene();

  picture = new PPMrast("ppm/wall.ppm");
//squares = new PPMrast("ppm/square.pgm");
  eye = new PPMrast("ppm/eye4.ppm");
cylwall = new PPMrast("ppm/cylwall.ppm");
sphwall = new PPMrast("ppm/sphwall.ppm");
unitleft =    new v3( -0.5 , 0.0 , 0.5 );
unitupleft =  new v3(0.5 , -1.  , 0.5 );
unitright =   new v3(  0.5 , 0.0 ,  0.5 );
unitupright = new v3( -0.5 , -1.  , 0.5 );

backleft =    new v3( -0.5 , 0.0 , -0.5 );
backupleft =  new v3(-0.5 , 1.  , 0.5 );
backright =   new v3(  0.5 , 0.0 ,  -0.5 );
backupright = new v3( 0.5 , 1.  , 0.5 );
  //radmult = sqrt(5.0) / 2.0;
  radmult = 1.0;
  pitch = 0.25;

// pyramid:
  surfprop = new SurfProps(1.0, 0.0, 0.0, 0.0,Pixmap_color, picture, Constant_bump, NULL);
  plane = new Plane(v3(0.0, 0.0, 0.),  unitupleft->unit(),   unitleft->unit());
  surface = new Surface(Plane_tag, plane, surfprop);
  p2 = new Plane(v3(0.0, 0.0, 0.0), unitright->unit(),  unitupright->unit());
  s2 = new Surface(Plane_tag, p2, surfprop);
  csgnode = new CSGnode(csg_op_intersection, csg_prim, csg_prim, surface, s2);

  plane = new Plane(v3(0.0, 0.0, 0.),  backupleft->unit(),   backleft->unit());
  surface = new Surface(Plane_tag, plane, surfprop);
  p2 = new Plane(v3(0.0, 0.0, 0.0), backright->unit(),  backupright->unit());
  s2 = new Surface(Plane_tag, p2, surfprop);
  csg2 = new CSGnode(csg_op_intersection, csg_prim, csg_prim, surface, s2);

  csgnode = new CSGnode(csg_op_intersection, csg_complex, csg_complex, csg2, csgnode);
  //roof:
  //plane = new Plane(v3(.0, 1.0, 0.),  v3(2.0, 0.0, .0), v3(.0, 0., 2.));
  //surface = new Surface(Plane_tag, plane, surfprop);
  sphere = new Sphere(v3(0., .0, 0.), 1.5 + (radmult * (pitch)) , v3(.0, 1.0, .0), v3(.0, 0., 1.));
  surface = new Surface(Sphere_tag, sphere, surfprop);
  notnode = new CSGnode(csg_op_not, csg_prim, csg_prim, surface,NULL);
  pyramid = new CSGnode(csg_op_intersection, csg_complex, csg_complex, notnode,
	 			csgnode);
//end pyramid


//sky:
//  a_color = new v3(0., 0., 1.0);
//  surfprop = new SurfProps(1.0, 0.0, 0.0, 0.0,Constant_color, a_color, Constant_bump, NULL);
//  plane = new Plane(v3(0.0, -20.0, 0.), v3(-2.0, 0.0, .0), v3(.0, 0., 2.));
//  surface = new Surface(Plane_tag, plane, surfprop);
//  csgnode = new CSGnode(csg_op_union, csg_prim, csg_complex, surface, csgnode);
//end sky

//ground:
  //a_color = new v3(0.5, 0.5, .4);
  //surfprop = new SurfProps(0.6, 0.0, 0.0, 0.4,Constant_color, a_color, Pixmap_bump, squares);
  //plane = new Plane(v3(0.0, 2.4, 0.), v3(1.0, 0., .0), v3(.0, 0., 1.));
  //surface = new Surface(Plane_tag, plane, surfprop);
  //csgnode = new CSGnode(csg_op_union, csg_prim, csg_complex, surface,
		//csgnode);


  pitch = 0.05;
//Shells:
  surfprop = new SurfProps(1.0, 0.0, 0.0, 0.0,Pixmap_color, cylwall, Constant_bump, NULL);
  for (shelloop = 0; shelloop < 50 ; shelloop++)
  {
	// inner:
  	cyl = new Cylinder(v3(0., -1.6, 0.), v3(0.0, 1.0, 0.0),  v3(.0, 0.0, 1.0) , 0.6 +         2.0 * pitch * shelloop);
  	s2 = new Surface(Cylinder_tag, cyl, surfprop);
	// outer:
  	cyl = new Cylinder(v3(0., -1.6, 0.), v3(0.0, 1.0, 0.0),  v3(.0, 0.0, 1.0) , 0.6 + pitch * 6.0 / 4.0 + 2.0 * pitch * shelloop);
  	surface = new Surface(Cylinder_tag, cyl, surfprop);
  	notnode = new CSGnode(csg_op_not, csg_prim, csg_prim, surface,NULL);

  	notshell = new CSGnode(csg_op_union, csg_prim, csg_complex, s2, notnode); 

 	pyramid = new CSGnode(csg_op_intersection, csg_complex, csg_complex, notshell, pyramid);
  }

  pitch = 0.25;
//Shells:
  surfprop = new SurfProps(1.0, 0.0, 0.0, 0.0,Pixmap_color, sphwall, Constant_bump, NULL);
  for (shelloop = 0; shelloop < 10 ; shelloop++)
  {
	// inner:
  	sphere = new Sphere(v3(0., 0.0 , 0.),1.5                          + 2.0 * radmult * pitch * shelloop, v3(.0, 1.0, .0), v3(.0, 0., 1.));
  	s2 = new Surface(Sphere_tag, sphere, surfprop);
	// outer:
  	sphere = new Sphere(v3(0., 0.0 , 0.), 1.5 + pitch * radmult * 7.0 / 4.0  +  2.0 * radmult * pitch * shelloop, v3(.0, 1.0, .0), v3(.0, 0., 1.));
  	surface = new Surface(Sphere_tag, sphere, surfprop);
  	notnode = new CSGnode(csg_op_not, csg_prim, csg_prim, surface,NULL);

  	notshell = new CSGnode(csg_op_union, csg_prim, csg_complex, s2, notnode); 

  	pyramid = new CSGnode(csg_op_intersection, csg_complex, csg_complex, notshell, pyramid);
  }
  csgnode = pyramid;

  //eye:
  surfprop = new SurfProps(1.0, 0.0, 0.0, 0.0,Pixmap_color, eye, Constant_bump, NULL);
  sphere = new Sphere(v3(0., 1., 0.),0.8, v3(.0, 1.0, .0), v3(.0, 0., 1.));
  surface = new Surface(Sphere_tag, sphere, surfprop);
  csgnode = new CSGnode(csg_op_union, csg_complex, csg_prim, csgnode, surface);

  scene->csgnode = csgnode;
  csgnode->print(1);
  lightlist->Print();
  global_lightlist = new LightList(lightlist);
// end sloppy init
  triples_per_line = 0;
  fprintf(Rasterfile, "P6\n# Made by Dirt, (c) Sean Brennan 1994\n");
  fprintf(Rasterfile, "%d %d\n", SCREEN_WIDTH, SCREEN_HEIGHT);
  fprintf(Rasterfile, "%d\n", 255); // color values
  while ((a_ray = a_screen.getray()) != NULL)
    {
      a_ray->direction = WORLD_TRANSFORM * a_ray->direction;
      a_photon = scene->Testray(a_ray);
      if (a_photon->x > 1.0)
	a_photon->x = 1.0;
      if (a_photon->y > 1.0)
	a_photon->y = 1.0;
      if (a_photon->z > 1.0)
	a_photon->z = 1.0;
      fputc((int) (a_photon->x * 255.0), Rasterfile);
      fputc((int) (a_photon->y * 255.0), Rasterfile);
      fputc((int) (a_photon->z * 255.0), Rasterfile);
    }
}
  
